import Fs from '@supercharge/fs';
import { Collect } from '@supercharge/collections';
import { resolveDefaultImport } from '@supercharge/goodies';
import { Application as Craft } from '@supercharge/console';
import { HandleExceptions, LoadConfiguration, LoadEnvironmentVariables, RegisterServiceProviders, BootServiceProviders } from '../bootstrappers/index.js';
export class ConsoleKernel {
    /**
     * Stores the internal configuration.
     */
    meta;
    /**
     * Create a new console kernel instance.
     */
    constructor(app) {
        this.meta = { app };
    }
    /**
     * Create a new console kernel instance for the given `app`.
     */
    static for(app) {
        return new this(app);
    }
    /**
     * Returns the application instance.
     */
    app() {
        return this.meta.app;
    }
    /**
     * Returns the list of application bootstrappers.
     */
    bootstrappers() {
        return [
            HandleExceptions,
            LoadEnvironmentVariables,
            LoadConfiguration,
            RegisterServiceProviders,
            BootServiceProviders
        ];
    }
    /**
     * Handle an incoming console command for the given `input`.
     */
    async run(input) {
        await this.bootstrap();
        await this.craft().run(input);
    }
    /**
     * Prepare the console application by running the configured bootstrappers.
     * This method doesn’t register configured commands in the application.
     * It prepares the console application which is useful for testing.
     */
    async prepare() {
        await this.app().bootstrapWith(this.bootstrappers());
        return this;
    }
    /**
     * Bootstrap the console application for Craft commands.
     */
    async bootstrap() {
        await this.prepare();
        await this.commands();
    }
    /**
     * Register the console commands to the application.
     */
    async commands() {
        //
    }
    /**
     * Register all of the commands in the given `directories`.
     */
    async loadCommandsFromPaths(...paths) {
        await Collect(paths)
            .unique()
            .filter(async (path) => await Fs.exists(path))
            .flatMap(async (path) => await Fs.allFiles(path))
            .map(async (commandFilePath) => await this.resolveCommandFrom(commandFilePath))
            .forEach(command => this.craft().add(command));
    }
    /**
     * Requires the given file from disk.
     */
    async resolveCommandFrom(commandFile) {
        const CommandCtor = await resolveDefaultImport(commandFile);
        return new CommandCtor();
    }
    /**
     * Returns a Craft console application instance.
     */
    craft() {
        if (!this.meta.craft) {
            this.meta.craft = new Craft(this.app());
        }
        return this.meta.craft;
    }
}
