import { SetOptional } from 'type-fest';
import { HttpError } from './http-error.js';
import { Application, ErrorHandler as ErrorHandlerContract, HttpContext, Logger, RenderableError, ReportableError, ViewEngine } from '@supercharge/contracts';
export declare class ErrorHandler implements ErrorHandlerContract {
    /**
     * The application instance.
     */
    protected readonly app: Application;
    /**
     * The list of error types to not report.
     */
    protected readonly ignoredErrors: ErrorConstructor[];
    /**
     * Stores the list of report callbacks.
     */
    protected readonly reportCallbacks: Array<(error: any, ctx: HttpContext) => Promise<void> | void>;
    /**
     * Create a new error handler instance.
     */
    constructor(app: Application);
    /**
     * The application’s logger instance.
     */
    logger(): Logger;
    /**
     * The application’s view renderer.
     */
    view(): ViewEngine;
    /**
     * Register the error handling callbacks. For example, to report error
     * upstream to an error tracking service, like Sentry or Bugsnag.
     */
    register(): void;
    /**
     * Tell the error handler to not report the `error` type.
     */
    ignore(error: ErrorConstructor): this;
    /**
     * Returns an array of errors that should not be reported.
     */
    dontReport(): ErrorConstructor[];
    /**
     * Determine whether to report the given `error`.
     */
    shouldNotReport(error: Error): boolean;
    /**
     * Register a reportable callback.
     */
    reportable(reportUsing: (error: any, ctx: HttpContext) => void | Promise<void>): ErrorHandler;
    /**
     * Returns the error’s context for logging.
     */
    errorContext(error: any): Record<string, any>;
    /**
     * Returns the default logging context.
     */
    context(_ctx: HttpContext): any;
    /**
     * Handle the given error.
     */
    handle(error: Error, ctx: HttpContext): Promise<void>;
    /**
     * Report an error.
     */
    report(error: Error, ctx: HttpContext): Promise<void>;
    /**
     * Determine whether the given `error` is implementing a `report` method and
     * that `report` method returns a truthy value, like a valid HTTP response.
     */
    errorReported(error: SetOptional<ReportableError, 'report'>, ctx: HttpContext): Promise<unknown>;
    /**
     * Render the error into an HTTP response.
     */
    render(error: Error, ctx: HttpContext): Promise<any>;
    /**
     * Determine whether the given `error` is implementing a `render` method and
     * that `render` method returns a truthy value, like a valid HTTP response.
     */
    errorRendered(error: SetOptional<RenderableError, 'render'>, ctx: HttpContext): Promise<unknown>;
    /**
     * Creates a JSON response depending on the app’s environment.
     */
    protected renderJsonResponse(error: HttpError, ctx: HttpContext): void;
    /**
     * Creates an HTML response depending on the app’s environment.
     */
    protected renderViewResponse(error: HttpError, ctx: HttpContext): Promise<void>;
    /**
     * Determine whether a view template file is missing for the given `error`.
     */
    protected isMissingTemplateFor(error: HttpError): Promise<boolean>;
    /**
     * Determine whether a view template file exists for the given `error`.
     */
    protected templateExistsFor(error: HttpError): Promise<boolean>;
    /**
     * Returns the view template file for the given `error`.
     */
    protected viewTemplateFor(error: HttpError): string;
    /**
     * Renders an HTML response containing details about the given `error`.
     */
    renderYouchResponse({ request, response }: HttpContext, error: HttpError): Promise<void>;
}
