import Youch from 'youch';
import { tap } from '@supercharge/goodies';
import { HttpError } from './http-error.js';
import { Collect } from '@supercharge/collections';
export class ErrorHandler {
    /**
     * The application instance.
     */
    app;
    /**
     * The list of error types to not report.
     */
    ignoredErrors;
    /**
     * Stores the list of report callbacks.
     */
    reportCallbacks;
    /**
     * Create a new error handler instance.
     */
    constructor(app) {
        this.app = app;
        this.ignoredErrors = [];
        this.reportCallbacks = [];
        this.register();
    }
    /**
     * The application’s logger instance.
     */
    logger() {
        return this.app.logger();
    }
    /**
     * The application’s view renderer.
     */
    view() {
        return this.app.make('view');
    }
    /**
     * Register the error handling callbacks. For example, to report error
     * upstream to an error tracking service, like Sentry or Bugsnag.
     */
    register() {
        //
    }
    /**
     * Tell the error handler to not report the `error` type.
     */
    ignore(error) {
        return tap(this, () => {
            this.ignoredErrors.push(error);
        });
    }
    /**
     * Returns an array of errors that should not be reported.
     */
    dontReport() {
        return [];
    }
    /**
     * Determine whether to report the given `error`.
     */
    shouldNotReport(error) {
        return this
            .dontReport()
            .concat(this.ignoredErrors)
            .some(ErrorConstructor => {
            return error instanceof ErrorConstructor;
        });
    }
    /**
     * Register a reportable callback.
     */
    reportable(reportUsing) {
        return tap(this, () => {
            this.reportCallbacks.push(reportUsing);
        });
    }
    /**
     * Returns the error’s context for logging.
     */
    errorContext(error) {
        if (typeof error.context === 'function') {
            return { ...error.context() };
        }
        return {};
    }
    /**
     * Returns the default logging context.
     */
    context(_ctx) {
        return {};
    }
    /**
     * Handle the given error.
     */
    async handle(error, ctx) {
        await this.report(error, ctx);
        await this.render(error, ctx);
    }
    /**
     * Report an error.
     */
    async report(error, ctx) {
        if (this.shouldNotReport(error)) {
            return;
        }
        if (await this.errorReported(error, ctx)) {
            return;
        }
        const handled = await Collect(this.reportCallbacks).any(async (reportCallback) => {
            return await reportCallback(error, ctx);
        });
        if (handled) {
            return;
        }
        const context = {
            ...this.context(ctx),
            ...this.errorContext(error),
            error,
        };
        this.logger().error(error.message, context);
    }
    /**
     * Determine whether the given `error` is implementing a `report` method and
     * that `report` method returns a truthy value, like a valid HTTP response.
     */
    async errorReported(error, ctx) {
        if (typeof error.report !== 'function') {
            return false;
        }
        return !!(await error.report(error, ctx));
    }
    /**
     * Render the error into an HTTP response.
     */
    async render(error, ctx) {
        if (await this.errorRendered(error, ctx)) {
            return;
        }
        const httpError = HttpError.wrap(error);
        if (ctx.request.wantsJson()) {
            return this.renderJsonResponse(httpError, ctx);
        }
        if (this.app.env().isProduction()) {
            return this.renderJsonResponse(httpError, ctx);
        }
        return await this.renderViewResponse(httpError, ctx);
    }
    /**
     * Determine whether the given `error` is implementing a `render` method and
     * that `render` method returns a truthy value, like a valid HTTP response.
     */
    async errorRendered(error, ctx) {
        if (typeof error.render !== 'function') {
            return false;
        }
        return await error.render(error, ctx);
    }
    /**
     * Creates a JSON response depending on the app’s environment.
     */
    renderJsonResponse(error, ctx) {
        const { message, stack, status: statusCode } = error;
        this.app.env().isProduction()
            ? ctx.response.status(statusCode).payload({ message, statusCode })
            : ctx.response.status(statusCode).payload({ message, statusCode, stack });
    }
    /**
     * Creates an HTML response depending on the app’s environment.
     */
    async renderViewResponse(error, ctx) {
        if (await this.isMissingTemplateFor(error)) {
            return await this.renderYouchResponse(ctx, error);
        }
        // @ts-expect-error
        if (typeof ctx.response.view === 'function') {
            // @ts-expect-error
            return await ctx.response.status(error.status).view(this.viewTemplateFor(error), { error });
        }
        ctx.response.status(error.status).payload(`<h1>${error.message}</h1>`);
    }
    /**
     * Determine whether a view template file is missing for the given `error`.
     */
    async isMissingTemplateFor(error) {
        return !await this.templateExistsFor(error);
    }
    /**
     * Determine whether a view template file exists for the given `error`.
     */
    async templateExistsFor(error) {
        return await this.view().exists(this.viewTemplateFor(error));
    }
    /**
     * Returns the view template file for the given `error`.
     */
    viewTemplateFor(error) {
        return `errors/${error.status}`;
    }
    /**
     * Renders an HTML response containing details about the given `error`.
     */
    async renderYouchResponse({ request, response }, error) {
        response.payload(await new Youch(error, request.req()).toHTML({})).status(error.status);
    }
}
