import Os from 'node:os';
import Set from '@supercharge/set';
export class ShutdownSignalListener {
    /**
     * Stores the application instance.
     */
    app;
    /**
     * Stores the shutdown signals to listen on.
     */
    shutdownSignals;
    /**
     * Stores the shutdown callback.
     */
    onShutdownCallbacks;
    /**
     * Invoke callback and exit process
     */
    kill = async function (signal) {
        try {
            await Promise.race([
                ...this.onShutdownCallbacks,
                new Promise((resolve) => {
                    setTimeout(resolve, 3000);
                }),
            ]);
            process.exit(Os.constants.signals[signal]);
        }
        catch (error) {
            process.exit(1);
        }
    }.bind(this);
    /**
     * Create a new instance.
     */
    constructor(app) {
        this.app = app;
        this.onShutdownCallbacks = [];
        this.shutdownSignals = ['SIGINT', 'SIGTERM'];
    }
    /**
     * Assign the given shutdown `callback` that runs when receiving a shutdown signal.
     */
    onShutdown(callback) {
        if (typeof callback !== 'function') {
            throw new TypeError(`Invalid argument: you must provide a callback function to "onShutdown", received "${typeof callback}"`);
        }
        this.onShutdownCallbacks.push(callback);
        return this;
    }
    /**
     * Close on SIGINT AND SIGTERM SIGNALS
     */
    listen(...signals) {
        this.runShutdownCallbackOn(...signals);
        /**
         * Log the uncaught exception error.
         */
        process.on('uncaughtExceptionMonitor', (error) => {
            this.app.logger().alert(error.message, { error });
        });
    }
    /**
     * Register the shutdown handler to run for the given `signals`.
     */
    runShutdownCallbackOn(...signals) {
        this.shutdownSignals = Set
            .from(...signals)
            .concat(this.shutdownSignals)
            .toArray();
        this.shutdownSignals.forEach(signal => {
            // eslint-disable-next-line @typescript-eslint/no-misused-promises
            process.once(signal, this.kill);
        });
    }
    /**
     * Remove the event listeners for shutdown signals and
     * reset the shutdown callback to a no-op function.
     */
    cleanup() {
        this.shutdownSignals.forEach(signal => {
            // eslint-disable-next-line @typescript-eslint/no-misused-promises
            process.removeListener(signal, this.kill);
        });
        this.onShutdownCallbacks = [];
        return this;
    }
}
